#include <types.h>

#include <timer.h>
#include <vdp.h>
#include <vdp_dma.h>

#include <vdp_tile.h>
#include <vdp_pal.h>

#include "utils/anim.h"
#include "utils/unpack.h"

#ifdef DEBUG
#include <tools.h>
#include "utils/debug.h"
#endif

#include "utils/sprite.h"

/********* TO DO **********
 *
 * - Anim UpdateOne (behaviors)
 * - Anim Kill
 *
 **************************/

struct AnimatedSprite	Anim[MAX_ANIMS];

/********************************** PRIVATE *****************************/
/******* ANIM_FINDFREE ******************
 * find the first unactive animSprite
 ****************************************/
static u8 Anim_FindFree( u8 searchFrom , u8 searchTo)
{
	u8 i;

	for(i=searchFrom; i<searchTo; i++)
	{
		if (Anim[i].active == FALSE)
		{
				return (i);
		}
	}

	return (0xFF);
}

/***** ANIM_UPDATEON **********
 * update an animSprite (frame, ticks)
 ****************************/
static void Anim_UpdateOne( u8 idxSpr )
{
	//u16 posx, posy, spIdx;
	struct str_Frame *frame;

	if (Anim[idxSpr].active == FALSE)	return;

	frame = (struct str_Frame *) Anim[idxSpr].AnimLib->Frames[ Anim[idxSpr].curFrame ];
	Anim[idxSpr].curTick++;

	if ( Anim[idxSpr].curTick >= frame->Timer )
	{
		Anim[idxSpr].curFrame = frame->Next;
		Anim[idxSpr].curTick = 0;
	}
}

/******************************** PUBLIC *********************************/

/********* ANIM_RAZ ********
 * free all animSprites
 ***************************/
void Anim_Init( )
{
	u8 i;

	for(i=0; i<MAX_ANIMS; i++)
			Anim_Delete(i);
}

/******* ANIM_LOAD ************
 * load the tiles of an animLib
 ******************************/
u16 Anim_Load( struct str_Anim *AnimLib, u16 idxTile)
{
	u32 addr;

	if (AnimLib->compressSize > 0)
	{
		addr = idxTile*32;
		unpack(AnimLib->sprite_data, GFX_WRITE_VRAM_ADDR(addr), AnimLib->compressSize);
	}
	else
	{
		VDP_DoVRamDMACopy(  (u16) AnimLib->sprite_data, 32*idxTile, AnimLib->nbTiles*32 );
	}

	return ( idxTile + AnimLib->nbTiles );
}

/****** ANIM_NEW *********
 * init a new animSprite
 *************************/
u8 Anim_New( u8 IdCode, struct str_Anim *AnimLib, u16 firstTile, u16 posX, u16 posY, u8 pal)
{
	u8 i;
	i = Anim_FindFree( 0, MAX_ANIMS );

	if (i == 0xFF)	return (0xFF);

	Anim[i].active = TRUE;
	Anim[i].IDCode = IdCode;
	Anim[i].bhvIdx = 0;
	Anim[i].idxTile = firstTile;
	Anim[i].curFrame = 0;
	Anim[i].curTick = 0;
	Anim[i].x = posX;
	Anim[i].y = posY;
	Anim[i].palIdx = pal;
	Anim[i].flip = 0;
	Anim[i].data = 0;//data;
	Anim[i].AnimLib=AnimLib;

	return (i);
}

/********ANIM_DELETE ******
 * inactive an animSprite
 **************************/
void Anim_Delete( u8 idxSpr )
{
	Anim[idxSpr].active = FALSE;
}

/***** ANIM_UPADE **********
 * update all the animSprite
 * and draw them
 ****************************/
void Anim_Update(  )
{
	Anim_Update1( );
	Anim_Update2( );
}

void Anim_Update1(  )
{
	u8 i;

	for(i=0; i<MAX_ANIMS; i++)
	{
//		Script_UpdateOne( i ); // update position and activity
		Anim_UpdateOne( i ); // update anim
	}
}

void Anim_Update2(  )
{
	u8 i;

	Sprite_Init( FALSE );

	for(i=0; i<MAX_ANIMS; i++)
	{
		Sprite_Add( i ); // add sprite
	}

	Sprite_Flush( ); // show them
}

/***** ANIM_JUMPTO ********
 * jump to a specific frame
 **************************/
BOOL	Anim_JumpTo( u8 idxSpr, u8 toFrame)
{
	if ( Anim[idxSpr].active == FALSE) return FALSE;

	if ( toFrame >= Anim[idxSpr].AnimLib->nbFrames)	return FALSE;


	Anim[idxSpr].curFrame = toFrame;
	Anim[idxSpr].curTick = 0;

	return TRUE;
}
