#include <types.h>
#include <vdp.h>
#include <vdp_spr.h>

#include <utils/anim.h>
#include <utils/debug.h>

/********* TO DO **********
 *
 * - multi sprite
 *
 **************************/

static u8	sprNum;


/********************************** PRIVATE *****************************/
u16  sprites_list[40*5]; /* 40 sprites max */

/*************** Sprite_Define ******************
 * Definition of the sprite                     *
 * number = number of the sprite                *
 * posx, posy =position of the sprite           *
 * size = in hexa                               *
 *        ex: 0xb00 = 3*4 (b=1011)              *
 * from =                                       *
 *      bit 15= priority                        *
 *      bit 14-13 = palette                     *
 *      bit 12 = vertical flipping              *
 *      bit 11 = horizontal flipping            *
 *      bit 10-0 = tile number                  *
 ************************************************/
static BOOL Sprite_Define(u8 number,u16 posx,u16 posy,u16 size,u16 from)
{
        /* As you can see, all is based on a u32 array         */
        /* It's my choice, but you can find something else      */
        u16 loc;

        if ( number >= 40 )	return FALSE;

        loc=5*number;
        sprites_list[loc]=posy&0x03FF;
        sprites_list[loc+1]=size&0x0F00;
        sprites_list[loc+2]=number&0x007F;
        sprites_list[loc+3]=from&0xFFFF;
        sprites_list[loc+4]=posx&0x03FF;

        return TRUE;
}


/********************* Sprite_Delete ********************
 * Reset of the nbr first sprites                       *
 ********************************************************/
static void Sprite_Delete( u8 nbr)
{
        register u32 *pl;
        register volatile u16 *pw;
        u16 i;

				if ( nbr >= 40 )	return;

				nbr*=5;

        pl = (u32 *) GFX_CTRL_PORT;
        *pl = GFX_WRITE_VRAM_ADDR(SLIST);

        pw = (u16 *) GFX_DATA_PORT;

        for (i= 0 ; i < nbr ; i++)
                *pw = 0x0000;
}

/*********************** Sprite_Show ********************
 *  Show the sprites                                    *
 *  from = number of the first sprite to show           *
 *  to   =   "    "   "  last    "    "   "             *
 ********************************************************/
static void Sprite_Show(u8 from, u8 to)
{
    register u32 *pl;
    register u16 *pw;
    register u16 i;

    /* Sprite table begins at (see GENESIS.H)  */
    /* Note: We set this address using         */
    /*       reg. 85 in the init_gfx routine. */
    pl = (u32 *) GFX_CTRL_PORT;
    *pl = GFX_WRITE_VRAM_ADDR(SLIST);

    pw = (u16 *) GFX_DATA_PORT;

    for ( i = from ; i < to ; i++ )
    {
            /* Set the y position */
            *pw = sprites_list[5*i];

            /* We need to set the size and number of the */
            /* sprite we want to show.  0=last, 1=first, */
            /* 2=second, etc...                          */
            /* 0x500 = 2x2 tile sprite                   */
            /* 0 = last sprite                           */
            if (i == (to-1))
            	*pw = sprites_list[5*i +1];
            else
            	*pw = (sprites_list[5*i +1] | (i+1));

            /* bit 15= priority                          */
            /* bit 14-13 = palette                       */
            /* bit 12 = vertical flipping                */
            /* bit 11 = horizontal flipping              */
            /* bit 10-0 = tile number                    */
            /* The genesis shows sprites x then y.       */
            /* Note: Look at the tile data.              */
            *pw = sprites_list[5*i +3];

            /* x position */
            *pw = sprites_list[5*i +4];
    }
}


/******************************** PUBLIC *********************************/


/****** SPRITE_ADD ******************
 * add current frame of an AnimSpr
 * to Genesis sprites list
 *************************************/
void Sprite_Add( u8 spriteRef )
{
	u16 posx;
	u16 posy;
	u16 size;
	u16 from; // = 0x0004;
	u16 idxTile;
	struct str_Frame *frame;

	u16 curSpr;

	if 	(Anim[ spriteRef ].active == FALSE)	return;

	frame = (struct str_Frame *) Anim[spriteRef].AnimLib->Frames[ Anim[spriteRef].curFrame ];
	idxTile = Anim[spriteRef].idxTile + frame->Sprites[0].Idx*Anim[spriteRef].AnimLib->sprite_width*Anim[spriteRef].AnimLib->sprite_height;

	size = (Anim[spriteRef].AnimLib->sprite_width-1);
	size <<= 2;
	size |= (Anim[spriteRef].AnimLib->sprite_height-1);
	size <<= 8;

//	from = 0x0001;
	from = 0x0000; // priority = 0
	from <<=2;
	from |= Anim[ spriteRef ].palIdx;
	from <<=2;
	from |= Anim[ spriteRef ].flip;
	from <<=11;
	from |= (idxTile&0x07FF);


	for( curSpr=0; curSpr < frame->nbSprites; curSpr++)
	{
		posx = Anim[ spriteRef ].x + frame->Sprites[curSpr].X;
		posy = Anim[ spriteRef ].y + frame->Sprites[curSpr].Y;

		Sprite_Define(sprNum++,posx,posy,size,from);
	}
}


/****** SPRITE_INIT ******************
 * reset sprite data
 *************************************/
void Sprite_Init( BOOL bFull )
{
	if (bFull)	Sprite_Delete(sprNum);
	sprNum = 0;
}

/****** SPRITE_SHOW ******************
 * show all the sprites
 *************************************/
void Sprite_Flush( )
{
	if (sprNum)	Sprite_Show(0,sprNum);
}


